@@ -454,6 +454,55 @@ def Save_Mol2(molecule, filename=None):
 
 sys.modules['__main__'].__dict__["Save_Mol2"] = Save_Mol2
 
+def Save_Gro(molecule, filename):
+    towrite = "Generated By Xponge\n"
+    molecule.atoms = []
+    for res in molecule.residues:
+        molecule.atoms.extend(res.atoms)
+    molecule.residue_index = {molecule.residues[i]: i for i in range(len(molecule.residues))}
+    
+    boxlength = [0, 0, 0]
+    maxi = [-float("inf"), -float("inf"), -float("inf")]
+    mini = [float("inf"), float("inf"), float("inf")]
+    for atom in molecule.atoms:
+        if atom.x > maxi[0]:
+            maxi[0] = atom.x
+        if atom.y > maxi[1]:
+            maxi[1] = atom.y
+        if atom.z > maxi[2]:
+            maxi[2] = atom.z
+        if atom.x < mini[0]:
+            mini[0] = atom.x
+        if atom.y < mini[1]:
+            mini[1] = atom.y
+        if atom.z < mini[2]:
+            mini[2] = atom.z
+
+    towrite += "%d\n"%len(molecule.atoms)
+    for i, atom in enumerate(molecule.atoms):
+        residue = atom.residue
+        if not GlobalSetting.nocenter:
+            x = atom.x - mini[0] + 3
+            y = atom.y - mini[1] + 3
+            z = atom.z - mini[2] + 3
+        else:
+            x, y, z = atom.x. atom.y, atom.z
+         
+        towrite += "%5d%-5s%5s%5d%8.3f%8.3f%8.3f\n"%(molecule.residue_index[residue] + 1, residue.name, atom.name, i+1, x/10, y/10, z/10)
+    if molecule.box_length is None:
+        boxlength[0] = maxi[0] - mini[0] + 6
+        boxlength[1] = maxi[1] - mini[1] + 6
+        boxlength[2] = maxi[2] - mini[2] + 6
+        molecule.box_length = [boxlength[0], boxlength[1], boxlength[2]]
+    else:
+        boxlength[0] = molecule.box_length[0]
+        boxlength[1] = molecule.box_length[1]
+        boxlength[2] = molecule.box_length[2]
+    towrite += "%8.3f %8.3f %8.3f"%(molecule.box_length[0] /10, molecule.box_length[1]/10, molecule.box_length[2]/10)
+    with open(filename, "w") as f:
+        f.write(towrite)
+    
+sys.modules['__main__'].__dict__["Save_Gro"] = Save_Gro
 
 def Save_NPZ(molecule, filename=None):
     import numpy as np
