@@ -1,4 +1,5 @@
 from . import *
+import sys
 
 def _get_rotate_matrix(r0, angle):
     cost = np.cos(angle)
@@ -120,7 +121,7 @@ def Impose_Bond(molecule, atom1, atom2, length):
         crd[molecule.atom_index[atom2]] += (1/3)**(0.5)
         r0 = crd[molecule.atom_index[atom2]] - crd[molecule.atom_index[atom1]]
         L0 = np.linalg.norm(r0)
-    dr = (L0 - length) * r0
+    dr = (length/L0 - 1) * r0
     crd[atom2_friends] += dr
     for atom in molecule.atoms:
         i = molecule.atom_index[atom]
@@ -320,6 +321,8 @@ def Molecule_Add(self, other):
         resB = new_molecule2.residues[0]
         for res in new_molecule2.residues:
             new_molecule.Add_Residue(res)
+        for reslink in new_molecule2.residue_links:
+            new_molecule.Add_Residue_Link(reslink.atom1, reslink.atom2)
         if resA.type.tail and resB.type.head:
             atom1 = resA._name2atom[resA.type.tail]
             atom2 = resB._name2atom[resB.type.head]
@@ -350,6 +353,8 @@ def iMolecule_Add(self, other):
         resB = new_molecule2.residues[0]
         for res in new_molecule2.residues:
             self.Add_Residue(res)
+        for reslink in new_molecule2.residue_links:
+            self.Add_Residue_Link(reslink.atom1, reslink.atom2)
         if resA.type.tail and resB.type.head:
             atom1 = resA._name2atom[resA.type.tail]
             atom2 = resB._name2atom[resB.type.head]
@@ -372,7 +377,7 @@ def Muls(self, other):
             t += self
         return t
     else:
-        raise TypeError("unsupported operand type(s) for +: '%s' and '%s'"%(type(self), type(other)))
+        raise TypeError("unsupported operand type(s) for *: '%s' and '%s'"%(type(self), type(other)))
 
 def iMuls(self, other):
     if type(other) == int:
@@ -381,7 +386,7 @@ def iMuls(self, other):
             self += self
         return self
     else:
-        raise TypeError("unsupported operand type(s) for +: '%s' and '%s'"%(type(self), type(other)))
+        raise TypeError("unsupported operand type(s) for *: '%s' and '%s'"%(type(self), type(other)))
 
 ResidueType.__add__ = ResidueType_Add
 ResidueType.__radd__ = ResidueType_Add
@@ -478,4 +483,4 @@ Molecule.__ior__ = iMolecule_Or
 
 del ResidueType_Or
 del Molecule_Or
-del iMolecule_Or
\ No newline at end of file
+del iMolecule_Or
