@@ -5,10 +5,10 @@ AngleType = Generate_New_Bonded_Force_Type("angle", "1-2-3", {"k":float, "b":flo
 AngleType.Set_Property_Unit("k", "energy·angle^-2", "kcal/mol·rad^-2")
 AngleType.Set_Property_Unit("b", "angle", "rad")
 
-@Molecule.Set_Save_SPONGE_Input
-def write_angle(self, prefix, dirname):
+@Molecule.Set_Save_SPONGE_Input("angle")
+def write_angle(self):
     angles = []
-    for angle in self.bonded_forces["angle"]:
+    for angle in self.bonded_forces.get("angle",[]):
         order = list(range(3))
         if angle.k != 0:
             if self.atom_index[angle.atoms[order[0]]] >  self.atom_index[angle.atoms[order[-1]]]:
@@ -22,6 +22,4 @@ def write_angle(self, prefix, dirname):
         towrite = "%d\n"%len(angles)
         angles.sort(key = lambda x: list(map(int, x.split()[:3])))
         towrite += "\n".join(angles)        
-        f = open(os.path.join(dirname, prefix + "_angle.txt"),"w")
-        f.write(towrite)
-        f.close()
\ No newline at end of file
+        return towrite
\ No newline at end of file
